function [filenames, filepaths, idxOpts] = parsefileinputs(optList, filetype, varargin)
    % PARSEFILEINPUTS Parses the input and checks their validity.
    %   Returns the filenames and filepaths for the input files, as well as the index where the
    %   remaining options start.
    %
    % -----
    % Usage:
    %   [filenames, filepaths, idxOpts] = PARSEPOLARINPUTS (optList, filetype)
    %   [filenames, filepaths, idxOpts] = PARSEPOLARINPUTS (optList, filetype, ...)
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    import af_tools.utils.*

    % First ensures the option list and filetype is properly set
    validateattributes(optList, {'char', 'string', 'cell'}, {}, mfilename(), 'optList', 1);
    validateattributes(filetype, {'char', 'string'}, {'scalartext'}, mfilename(), 'filetype', 2);
    filetype = char(filetype);
    if filetype(1) ~= '.'
        error('MATLAB:parsefileinputs:invalidFileType', ...
              'The filetype must starts by a point (.dat, .m, .mat, etc). Found ''%s''!\n', ...
              filetype);
    end

    % Parse inputs
    hasNoInput = isempty(varargin) || isoption(optList, varargin{1});
    if hasNoInput
        % Prompt user to select files
        ext = ['*', filetype];
        [filenames, filepaths] = uigetfile(ext, 'Select all dat-files to aggregate', ...
                                           'MultiSelect', 'on');
        idxOpts = 1;
    else
        idxOpts = 2;

        % Check if filenames properly formatted
        fullFiles = varargin{1};
        validateattributes(fullFiles, {'char', 'cell'}, {'nonempty', 'vector'}, ...
                           mfilename(), 'inputFiles', 3);
        fullFiles = string(fullFiles);

        % ---------------------------------------
        % Look for files
        AllFiles = [];

        for i = 1:numel(fullFiles)

            file = appendextension(fullFiles(i), filetype); % Add extension

            dummy = dir(file);  % List files properly
            if isempty(dummy)
                warning('MATLAB:parsefileinputs:FileNotFound', ...
                        'Could not find file %s.', fullFiles(i));
            end

            AllFiles = [AllFiles; dummy];
        end
        if isempty(AllFiles)
            error('MATLAB:parsefileinputs:noFilesFound', ...
                  'Impossible to find ANY file matching the input arguments.');
        end

        % Output the proper file names and path
        filenames = cell(1, length(AllFiles));
        filepaths = cell(1, length(AllFiles));
        for i = 1:length(AllFiles)
            filenames{i} = AllFiles(i).name;
            filepaths{i} = AllFiles(i).folder;
        end

    end

end

% --------------------------------------------------------------------------------------------------
function bool = isoption(optList, var)
    % ISOPTION Returns true if var is contained in option list

    if iscell(var)
        bool = false;
    else
        bool = any(strcmpi(var, optList));
    end
end
