function Dat = formatairfoilcoord(varargin)
    % FORMATAIRFOILCOORD Load and re-format airfoil coordinates.
    %   The UIUC airfoil database from the University of Illinois gathers the coordinates for more
    %   than 1600 airfoils. Two different standards are used to represent the coordinates of the
    %   airfoil:
    %       - Selig: The first line is the name of the airfoil, and each line after that is a set of
    %                coordinates. The coordinates are ordered from the trailing edge, along the
    %                upper surface to the leading edge and back around the lower surface to trailing
    %                edge. This format is the most common one.
    %       - Lednicer: The first line is the name of the airfoil, the second one is the number of
    %                   points on each side and the lines after that are the coordinates. The
    %                   coordinates are given separately for the upper and lower surfaces, each from
    %                   the leading edge to the trailing edge.
    %
    %   Many engineering tools or application (such as XFOIL) rely on inputs in the Selig format
    %   only. This script was therefore created to reformat original data so they can be used easily
    %   with other tools.
    %
    %   Even though the _Selig_ format is the default option, the script allows also the user to
    %   reformat _Selig_ files in _Lednicer_ form if needed. It also comes with the option to refine
    %   the coordinates by adding points with a spline interpolation following the half-cosine rule
    %   (more points are the leading and trailing edge). If that option is selected, the airfoil
    %   will be defined using 50 points for each surface.
    %
    % Note:
    %   This function can be used on many files at once in order to speed-up the process.
    %
    % <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox">Documentation (README)</a>
    % <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox/-/issues">Report an issue</a>
    % -----
    %
    % Usage:
    %   DAT = FORMATAIRFOILCOORD prompts the user for all inputs, then converts the coordinates to
    %   the Selig format.
    %
    %   DAT = FORMATAIRFOILCOORD(INPUTFILES) converts all files specified in INPUTFILES.
    %
    %   DAT = FORMATAIRFOILCOORD(..., 'autosave', true) saves the resulting coordinates in a new
    %   dat-file.
    %
    %   DAT = FORMATAIRFOILCOORD(..., 'overwrite', true) saves the resulting coordinates by
    %   overwriting the original dat-file. If autosave is not specified but overwrite is true, the
    %   file will be overwritten anyway. If autosave is set to false and overwrite is true, the file
    %   will not be saved at all.
    %
    %   DAT = FORMATAIRFOILCOORD(..., 'refine', true) refines the input data by using a spline
    %   interpolation with 100 coordinates in total, spaced following the half-cosine rule.
    %   False by default.
    %
    %   DAT = FORMATAIRFOILCOORD(..., 'outputFormat', 'Lednicer') select the output format (Selig or
    %   Lednicer). Selig by default.
    %
    % Inputs:
    %   inputFiles : Files to select (ex: 'dir/file', 'subdir/*' , 'dir/*0012*', {'file1','file2'})
    %
    % Output:
    %   Dat : A structure for the results for each input file.
    %         - Dat.file : original filename
    %         - Dat.format : Format of the output coordinates
    %         - Dat.airfoil : airfoil name, parsed from the input file
    %         - Dat.nPoints : Number of coordinates
    %         - Dat.coord : whole airfoil coordinates, with 0 <= X <= 1
    %         - Dat.upper : airfoil upper surface coordinates.
    %         - Dat.lower : airfoil lower surface coordinates.
    %
    % Example:
    %   FORMATAIRFOILCOORD
    %   FORMATAIRFOILCOORD('data/*0012*')
    %   FORMATAIRFOILCOORD({'data/0012_re1e5', 'data/0012_re1e6'})
    %   FORMATAIRFOILCOORD('test_data/*', 'autosave', true)
    %   FORMATAIRFOILCOORD('test_data/*', 'overwrite', true)
    %   FORMATAIRFOILCOORD('test_data/*', 'refine', true)
    %   FORMATAIRFOILCOORD('test_data/*', 'outputFormat', 'Lednicer')
    %
    % See also: NACAAIRFOIL.
    %
    % -----
    % UIUC Airfoil Database: https://m-selig.ae.illinois.edu/ads/coord_database.html
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    narginchk(0, 8);

    % Import other functions from this package
    import af_tools.utils.*

    % Constants and defaults
    OPTION_LIST = {'autosave', 'overwrite', 'refine', 'outputFormat'};
    FILETYPE = '.dat';
    REFINED_SPACING = 'halfcosine';
    REFINED_NPOINTS = ceil((100 + 1) / 2);

    % Parse inputs
    [allFileNames, fullpaths, idxOpts] = parsefileinputs(OPTION_LIST, FILETYPE, varargin{:});
    [autosave, overwrite, refine, outputFormat] = parseoptioninputs(varargin{idxOpts:end});

    % Convert filenames to string array
    allFileNames = string(allFileNames);

    % -------------------------------------------
    % Convert all files

    % Number of files found
    nbFiles = length(allFileNames);

    % Initialize structures
    Dat = struct('file', [], 'airfoil', [], 'format', [], 'nPoints', [], 'coord', [], ...
                 'upper', [], 'lower', []);

    for i = 1:nbFiles

        % Load file and start parsing
        % No matter the file type, line 1 should always be the airfoil name
        fileID = fopen(fullfile(fullpaths{i}, allFileNames{i}));
        tmpAirfoil = textscan(fileID, '%s', 1, 'Delimiter', '\n:', 'HeaderLines', 0);
        tmpCoord = cell2mat(textscan(fileID, '%f %f %*[^\n]', 'HeaderLines', 0));
        fclose(fileID);
        clear fileID;

        % Split airfoil into its upper and lower surfaces
        [Dat(i).upper, Dat(i).lower] = splitsurfaces(tmpCoord);

        % Refines coordinates by adding points if needed
        [Dat(i).upper, Dat(i).lower] = refinesurfaces(Dat(i).upper, Dat(i).lower, refine, ...
                                                      REFINED_SPACING, REFINED_NPOINTS);

        % Fromats according to desired output
        switch outputFormat
            case 'Selig'
                [Dat(i).coord, Dat(i).array] = selig(Dat(i).upper, Dat(i).lower);
            case 'Lednicer'
                [Dat(i).coord, Dat(i).array] = lednicer(Dat(i).upper, Dat(i).lower);
        end

        % Output structure
        Dat(i).path = fullpaths;
        Dat(i).file = allFileNames{i};
        Dat(i).airfoil = char(cellstr(tmpAirfoil{:}));
        Dat(i).format = outputFormat;

    end

    % -------------------------------------------
    % Save results automatically

    if autosave

        for i = 1:length(Dat)

            filename = split(Dat(i).file, '.dat');
            if ~overwrite
                filename = [filename{1}, '-', outputFormat];
            end
            filename = appendextension(filename, '.dat');
            filename = fullfile(Dat(i).path, filename);
            header = Dat(i).airfoil;
            savetodat(filename, header, Dat(i).array);
        end
    end

    % Cleanup
    Dat = rmfield(Dat, 'array');

end

% --------------------------------------------------------------------------------------------------
function [autosave, overwrite, refine, outputFormat] = parseoptioninputs(varargin)
    % PARSEOPTIONINPUTS Parses the options and checks their validity

    import af_tools.utils.*

    % Constants and defaults
    DEFAULT_AUTOSAVE = false;
    DEFAULT_OVEWRITE = false;
    DEFAULT_REFINE = false;
    DEFAULT_FORMAT = 'Selig';
    ALLOWED_FORMATS = {'Selig', 'Lednicer'};

    % Option validator
    validLogical = @(x) validateattributes(x, {'logical'}, {'scalar'}, mfilename());
    validFormat = @(x) any(validatestring(x, ALLOWED_FORMATS, mfilename()));

    % Parse options
    p = inputParser;
    addParameter(p, 'autosave', DEFAULT_AUTOSAVE, validLogical);
    addParameter(p, 'overwrite', DEFAULT_OVEWRITE, validLogical);
    addParameter(p, 'refine', DEFAULT_REFINE, validLogical);
    addParameter(p, 'outputFormat', DEFAULT_FORMAT, validFormat);

    parse(p, varargin{:});
    autosave = p.Results.autosave;
    overwrite = p.Results.overwrite;
    refine = p.Results.refine;
    outputFormat = p.Results.outputFormat;

end

function [upper, lower] = splitsurfaces(data)
    % SPLITSURFACES Splits the whole airfoil into its upper and lower surfaces

    % Determine format
    if data(1, 1) > 1
        inputFormat = 'Lednicer';
    elseif data(1, 1) == 1
        inputFormat = 'Selig';
    else
        error('MATLAB:formatairfoilcoord:unknownInputFormat', ...
              ['The second line of the data file must either be 1 (Selig format) or '...
               'an integer larger than 1 (Lednicer).']);
    end

    % Split lower and upper surfaces
    switch inputFormat
        case 'Selig'
            idx = find(diff(data(:, 1)) < 0, 1, 'last');
            upper = data(1:idx + 1, :);
            upper = flipud(upper);
            lower = data(idx + 1:end, :);
        case 'Lednicer'
            nCoord = data(1, 1);
            upper = data(2:nCoord + 1, :);
            lower = data(nCoord + 2:end, :);
    end

end

function [newUpper, newLower] = refinesurfaces(upper, lower, refine, spacing, nPoints)
    % REFINESURFACES Refines the upper and lower surfaces by adding more points

    import af_tools.utils.spacedvector

    if refine && size(upper, 1) < nPoints
        xc = fliplr(spacedvector(spacing, nPoints))';

        newUpper(:, 1) = xc;
        newLower(:, 1) = xc;
        newUpper(:, 2) = interp1(upper(:, 1), upper(:, 2), xc, 'spline');
        newLower(:, 2) = interp1(lower(:, 1), lower(:, 2), xc, 'spline');
    else
        newUpper = upper;
        newLower = lower;
    end

end
