# Airfoil toolbox

_A list of functions and script used for analysis of airfoils and related data
with Matlab._

## Disclaimer

This toolbox was mainly developed under Matlab R2018a. The various functions
were not tested on older versions. A code compatibility analysis showed no
errors, so _in theory_ it should work properly with any other versions of Matlab
as well. If for some reason it does not work on your version, please [open an
issue][gitlab_issues].

The functions in this repository are intended to be used as a _Matlab package_
(see [Matlab documentation][matlabDoc_packages] for more details).

## Documentation

Each function contains a complete description in its preamble. Type `help
<function>` in Matlab command window to print it.

## Install and use

### Installation

1. Either clone this repository or download a `.zip` version of the source code.
2. Place the `+af_tools/` folder somewhere on your Matlab Path such as:
    - in you current project
    - in a folder that is already in Matlab's path (e.g. `/Documents/Matlab/`).
    - somewhere else, but then [explicitly add it to Matlab's Path][matlabDoc_path].

#### Remark

Unfortunately, Matlab packages have a few limitations. Hence, you should **avoid
changing the package name** if you do not know what you are doing.

- The `+` in the folder's name is required for Matlab to treat it as a package.
- Some functions of this package are interconnected (_e.g._, _nacaairfoil_ calls
  _nacacamber_). In that situation, Matlab forces the main function to re-import
  explicitly all sub-functions before using them, even if they are part of the
  same package ([people have complained][matlabPackgeIssue] about that design
  choice for quite some time, to no avail).
  If you rename the package, these links will be broken and Matlab will not find
  the connected functions.
- All functions that import other functions will have an import line at the
  beginning. If you rename the package, you must therefore update these imports.

### Use

Once the `+af_tools/` folder is placed somewhere where Matlab will find
it, you can start using its features.

- If you are only interested in a single function of the package, use

```matlab
import af_tools.<functionName>
functionName(args)
```

- If you want to import all functions, use

```matlab
import af_tools.*
xf2mat(args)
nacacamber(args)
...
```

- If you want to use functions on-the-fly without importing them, call them
  directly

```matlab
af_tools.xf2mat(args)
```

## Functions

- All functions can be used as standalone functions and give meaningful output
  on their own (they are not sub-routines used to perform intermediary tasks).
- All functions can be called without arguments. If that is the case, they will
  prompt the user to manually enter or select the few mandatory arguments. The
  rest will be set to default values.

List of functions:

- [xf2mat](#xf2mat): Aggregates multiple XFOIL or XFLR5 polar results into a single structure.
- [formatairfoilcoord](#formatairfoilcoord): Re-format airfoil coordinates
  datafile.
- [Airfoil generation](#airfoil-generation)
  - [nacacamber](#nacacamber): Generates the coordinates of the camberline for a NACA 4 or 5 digits airfoil.
  - [nacaairfoil](#nacaairfoil): Generates the full coordinates of a NACA 4 or 5 digits airfoil.
- [Polar manipulation](#polar-manipulation)
  - [extendpolar](#extendpolar): Extends polars to the full range of $`\alpha`$ ([-180, 180] deg).
  - [findstall](#findstall): Finds the stall point and returns the corresponding $`\alpha`$, $`C_l`$ and $`C_d`$.
  - [findzerolift](#findzerolift): Finds the zero-lift angle and the associate $`C_d`$.
  - [findcllinearrange](#findcllinearrange): Finds the linear range of the lift coefficient and returns the range and lift slope.
  - [plotpolars](#plotpolars): Plot various polars.

### xf2mat

Aggregates airfoil polars obtained using [XFOIL v6.99][xfoil] of [XFLR5
v6.55][xflr5] into a single Matlab structure.

This function can either aggregate multiple polar files into a single structure
(_e.g._ same airfoil at different Reynolds) or simply convert a single polar
data into a Matlab-ready structure.

```matlab
import af_tools.xf2mat
Polar = xf2mat
Polar = xf2mat(inputFiles, 'autosave', true)
Polar = xf2mat(inputFiles, 'trimAoas', true)
Polar = xf2mat(inputFiles, 'autosave', true, 'trimAoas', true)
```

| Input        | Example                                        | Default |
|------------- | ---------------------------------------------- | --------|
| `inputFiles` | `'data/*'`, `'data/*0012*'`, `'{'data/*0012_1*', 'data/*0012_2*'}` | `'*'`   |
|              |                                                |         |
| 'autosave'   | `false`, `true`                                | `false` |
| 'trimAoas'   | `false`, `true`                                | `false` |

The output is a structure with the following fields, where M is the number of
angles of attack and N is the number of input files.

| Field        | Data                                             | Size    |
|------------- | ------------------------------------------------ | --------|
| `origin`     | Program from which the polars came (XFOIL/XFLR5) | [1 x N] |
| `airfoil`    | Airfoil name (parsed from XFOIL/XFLR5 results)   | [1 x N] |
| `alpha`      | Angles of attacks                                | [M x N] |
| `reynolds`   | Reynolds numbers                                 | [1 x N] |
| `mach`       | Mach number                                      | [1 x N] |
| `nCrit`      | BL transition factor                             | [1 x N] |
| `cl`         | Lift coefficient                                 | [M x N] |
| `cd`         | Drag coefficient                                 | [M x N] |
| `cm`         | Moment coefficient                               | [M x N] |

### formatairfoilcoord

The [UIUC airfoil database][uiuc-af-db] from the University of Illinois gathers
the coordinates for more than 1600 airfoils. Two different standards are used
to represent the coordinates of the airfoil:

- _Selig_: The first line is the name of the airfoil, and each line after that
  is a set of coordinates. The coordinates are ordered from the trailing edge,
  along the upper surface to the leading edge and back around the lower surface
  to trailing edge (_e.g._, see
  [E205](https://m-selig.ae.illinois.edu/ads/coord/e205.dat)). This format is
  the most common one.
- _Lednicer_: The first line is the name of the airfoil, the second one is the
  number of points on each side and the lines after that are the coordinates.
  The coordinates are given separately for the upper and lower surfaces, each
  from the leading edge to the trailing edge (_e.g._, see [Clark
  Y](https://m-selig.ae.illinois.edu/ads/coord/clarky.dat)).

Many engineering tools or application (such as [XFOIL][xfoil]) rely on inputs in
the _Selig_ format only. This script was therefore created to reformat original
data so they can be used easily with other tools.

Even though the _Selig_ format is the default option, the script allows also the
user to reformat _Selig_ files in _Lednicer_ form if needed. It also comes with
the option to refine the coordinates by adding points with a spline
interpolation following the half-cosine rule (more points are the leading and
trailing edge). If that option is selected, the airfoil will be defined using 50
points for each surface.

```matlab
import af_tools.xf2mat
Af = formatairfoilcoord
Af = formatairfoilcoord(inputFiles, 'autosave', true)
Af = formatairfoilcoord(inputFiles, 'overwrite', true)
```

| Input          | Example                                        | Default |
|--------------- | ---------------------------------------------- | --------|
| `inputFiles` | `'data/*'`, `'data/*0012*'`, `'{'data/*0012_1*', 'data/*0012_2*'}` | `'*'`   |
|                |                                                |         |
| 'autosave'     | `false`, `true`                                | `false` |
| 'overwrite'    | `false`, `true`                                | `false` |
| 'refine'       | `false`, `true`                                | `false` |
| 'outputFormat' | `Selig`, `Lednicer`                            | `Selig` |

### Airfoil generation

The following functions use the same two main input arguments.

| Input     | Example                            | Default        |
|---------- | ---------------------------------- | -------------- |
| `digits`  | `'0012'`, `'24012'`                | -              |
| `nPoints` | `100`, `130`                       | `100`          |

#### nacacamber

Determines the camber line of a NACA 4 or 5 digits airfoil.
The output is ordered from the leading edge to the trailing edge.

```matlab
import af_tools.nacacamber
[coord, gradY] = nacacamber
[coord, gradY] = nacacamber(digits)
[coord, gradY] = nacacamber(digits, nPoints)
[coord, gradY] = nacacamber(digits, nPoints, 'spacing', 'halfcosine')
```

| Options   | Example                            | Default        |
|---------- | ---------------------------------- | -------------- |
| 'spacing' | `halfcosine`, `cosine`, `linear`   | `'halfcosine'` |

#### nacaairfoil

Generates the full coordinates of a NACA 4 or 5 digits airfoil.
The output coordinates follow the most usual order. They start at the trailing
edge, along the upper surface to the leading edge and back around the lower
surface to trailing edge.

```matlab
import af_tools.nacaairfoil
[coord] = nacaairfoil
[coord] = nacaairfoil(digits)
[coord] = nacaairfoil(digits, nPoints)
[coord] = nacaairfoil(digits, nPoints, 'spacing', spacing, 'zerote', true)
[coord] = nacaairfoil(digits, nPoints, 'savedat', true)
```

| Options   | Example                            | Default        |
|---------- | ---------------------------------- | -------------- |
| 'spacing' | `halfcosine`, `cosine`, `linear`   | `'halfcosine'` |
| 'zerote'  | `true`, `false`                    | `true`         |
| 'savedat' | `true`, `false`                    | `true`         |

### Polar manipulation

The following functions all share the same main inputs arguments in the exact
same order. These arguments can be of two types:

  1. `Polar`: A Polar structure, as the one given by [xf2mat][xf2mat];
  2. `alpha`, `cl`, `cd`, `cm`: Arrays representing the different polars. _Some functions only allow the first two or three arrays._

Some functions then extend these inputs by using various optional arguments.

#### extendpolar

This function extends known $`C_l`$ and $`C_d`$ polars to the full range of
angle of attacks [-180; 180]. This is especially useful when studying rotors and
propellers in off-design configuration (with the Blade Element Momentum Theory
for instance).

```matlab
import af_tools.extendpolar
[alphaExt, clExt, cdExt] = extendpolar
[alphaExt, clExt, cdExt] = extendpolar(Polar)
[alphaExt, clExt, cdExt] = extendpolar(Polar, 'method', 'Viterna')
[alphaExt, clExt, cdExt] = extendpolar(Polar, 'limit', 'stall')
[alphaExt, clExt, cdExt] = extendpolar(alpha, cl, cd, 'method', 'Viterna')
```

| Options  | Example            | Default     |
|--------- | ------------------ | ----------- |
| 'method' | `'Viterna'`        | `'Viterna'` |
| 'limit'  | `'stall','limit'`  | `'last'`    |

#### plotpolars

Plots the following polars: $`C_l-\alpha`$ &nbsp;,&nbsp;&nbsp; $`C_d-\alpha`$
&nbsp;,&nbsp;&nbsp; $`C_m-\alpha`$ &nbsp;,&nbsp;&nbsp; $`C_l/C_d-\alpha`$
&nbsp;,&nbsp;&nbsp; $`C_l-C_d`$.

```matlab
import af_tools.plotpolars
plotpolars
plotpolars(Polar)
plotpolars(alpha, cl, cd, cm)
```

#### findstall

Finds the stall angle of attack and the associated $`C_l`$ and $`C_d`$.

```matlab
import af_tools.findstall
[alpha_s, cl_s, cd_s] = findstall
[alpha_s, cl_s, cd_s] = findstall(Polar)
[alpha_s, cl_s, cd_s] = findstall(alpha, cl, cd)
```

#### findzerolift

Finds the zero-lift angle of attack and the associated $`C_d`$.

```matlab
import af_tools.findzerolift
[alpha_zeroL, cd_zeroL] = findzerolift
[alpha_zeroL, cd_zeroL] = findzerolift(Polar)
[alpha_zeroL, cd_zeroL] = findzerolift(alpha, cl, cd)
```

#### findcllinearrange

Finds the range of angles of attack for which the $`C_l`$ can be considered
linear.

```matlab
import af_tools.findcllinearrange
[alphaRange, clRange, clSlope] = findcllinearrange
[alphaRange, clRange, clSlope] = findcllinearrange(Polar)
[alphaRange, clRange, clSlope] = findcllinearrange(alpha,cl)
```

## Licence

Copyright (c) 2022 [Thomas Lambert][tlambert], University of Liège.

All functions and scripts of the **af_tools** code are licensed under the
[Apache 2.0 Licence](LICENCE).

[xfoil]: http://web.mit.edu/drela/Public/web/xfoil/
[xflr5]: https://www.xflr5.tech/xflr5.htm
[uiuc-af-db]: https://m-selig.ae.illinois.edu/ads/coord_database.html
[tlambert]: <https://tlambert.xyz>
[gitlab_issues]: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
[matlabDoc_packages]: <https://www.mathworks.com/help/matlab/matlab_oop/scoping-classes-with-packages.html>
[matlabDoc_path]: <https://www.mathworks.com/help/matlab/matlab_env/what-is-the-matlab-search-path.html>
[matlabPackgeIssue]: https://www.mathworks.com/matlabcentral/answers/280234-misleading-wording-in-the-documentation-on-matlab-package-folders
