% TEST_EXTENDPOLAR Unitary tests for the extendpolar function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_extendpolar

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase,  ...
                @() af_tools.extendpolar(dummy, dummy, dummy, dummy, dummy, 'method', 'viterna', 'limit', 'stall'),  ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidInputsType(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;

    % 1. Polar (struct) or alpha (numeric)
    verifyError(testCase, @() af_tools.extendpolar(empty), 'MATLAB:parsepolarinputs:expectedNonempty');
    verifyError(testCase, @() af_tools.extendpolar(logi), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.extendpolar(char), 'MATLAB:parsepolarinputs:invalidType');

    % 2. cl (numeric)
    verifyError(testCase, @() af_tools.extendpolar(scal, logi), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.extendpolar(scal, char), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.extendpolar(scal, Struct), 'MATLAB:parsepolarinputs:invalidType');

    % 3. cd (numeric)
    verifyError(testCase, @() af_tools.extendpolar(scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.extendpolar(scal, scal, char), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.extendpolar(scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType');

end

function test_invalidInputSize(testCase)
    % Error if cl and cd do not have the same number of rows than alpha

    dummy = (1:20)';
    wrong = rand(8, 4);

    verifyError(testCase, @() af_tools.extendpolar(dummy, wrong, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows');
    verifyError(testCase, @() af_tools.extendpolar(dummy, dummy, wrong), 'MATLAB:parsepolarinputs:incorrectNumrows');

end

function test_invalidAlphaInput(testCase)
    % Error if alpha is not increasing (row directions)

    dummy = -(1:20)';

    verifyError(testCase, @() af_tools.extendpolar(dummy), 'MATLAB:parsepolarinputs:expectedIncreasing');

end

function test_invalidPolarInput(testCase)
    % Error if missing fields in the polar

    Dummy = dummysinglepolar();
    DummyAlpha = rmfield(Dummy, 'alpha');
    DummyCl = rmfield(Dummy, 'cl');
    DummyCd = rmfield(Dummy, 'cd');

    verifyError(testCase, @() af_tools.extendpolar(DummyAlpha), 'MATLAB:nonExistentField');
    verifyError(testCase, @() af_tools.extendpolar(DummyCl), 'MATLAB:nonExistentField');
    verifyError(testCase, @() af_tools.extendpolar(DummyCd), 'MATLAB:nonExistentField');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    Dummy = dummysinglepolar();
    wrongName = 'wrongoption';

    verifyError(testCase, @() af_tools.extendpolar(Dummy, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', 'Viterna', wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', 'Viterna', wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    Dummy = dummysinglepolar();
    empty = [];
    scal = 0;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', empty), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', scal), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', char), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', Struct), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', vect), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');

    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', empty), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', scal), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', char), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', Struct), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', vect), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');

end

% ------------------------------------------------------------------------------
function test_noStall(testCase)
    % Error if impossible to find the stall

    Polar = dummysinglepolar();
    Polar.cl = ones(size(Polar.cl));

    verifyError(testCase, @() af_tools.extendpolar(Polar), 'MATLAB:extendpolar:noStall');

end

function test_noZero(testCase)
    % Error if impossible to find the stall

    Polar = dummysinglepolar();
    % Shift alpha so it is all positive
    Polar.alpha = Polar.alpha(:) + abs(Polar.alpha(1)) + 0.1;

    verifyError(testCase, @() af_tools.extendpolar(Polar), 'MATLAB:extendpolar:noZero');

end

%% Test if function returns expected outputs

function test_correctAlphaSize(testCase)
    % Verify if the output alpha has the correct size
    % Should not re-expand polar that already covers the full range

    Polar = dummysinglepolar();
    stepAlpha = min(diff(Polar.alpha));
    fullRange = (-pi:stepAlpha:pi)';

    [alphaExt, clExt, cdExt] = af_tools.extendpolar(Polar);
    [alphaExt2, clExt2, cdExt2] = af_tools.extendpolar(alphaExt, clExt, cdExt);

    verifyEqual(testCase, alphaExt, fullRange, 'AbsTol', 1e-9);

    verifyEqual(testCase, alphaExt2, alphaExt);
    verifyEqual(testCase, clExt2, clExt);
    verifyEqual(testCase, cdExt2, cdExt);

end
