% TEST_FINDCLLINEARRANGE Unitary tests for the findcllinearrange function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_findcllinearrange

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase, @() af_tools.findcllinearrange(dummy, dummy, dummy), 'MATLAB:narginchk:tooManyInputs');

end

function test_invalidInputsType(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;

    % 1. Polar (struct) or alpha (numeric)
    verifyError(testCase, @() af_tools.findcllinearrange(empty), 'MATLAB:parsepolarinputs:expectedNonempty');
    verifyError(testCase, @() af_tools.findcllinearrange(logi), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.findcllinearrange(char), 'MATLAB:parsepolarinputs:invalidType');

    % 2. cl (numeric)
    verifyError(testCase, @() af_tools.findcllinearrange(scal, logi), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.findcllinearrange(scal, char), 'MATLAB:parsepolarinputs:invalidType');
    verifyError(testCase, @() af_tools.findcllinearrange(scal, Struct), 'MATLAB:parsepolarinputs:invalidType');

end

function test_invalidInputSize(testCase)
    % Error if cl and cd do not have the same number of rows than alpha

    dummy = (1:20)';
    wrong = rand(8, 4);

    verifyError(testCase, @() af_tools.findcllinearrange(dummy, wrong), 'MATLAB:parsepolarinputs:incorrectNumrows');

end

function test_invalidAlphaInput(testCase)
    % Error if alpha is not increasing (row directions)

    dummy = -(1:20)';

    verifyError(testCase, @() af_tools.findcllinearrange(dummy), 'MATLAB:parsepolarinputs:expectedIncreasing');

end

function test_invalidPolarInput(testCase)
    % Error if missing fields in the polar

    Dummy = dummysinglepolar();
    DummyAlpha = rmfield(Dummy, 'alpha');
    DummyCl = rmfield(Dummy, 'cl');
    DummyCd = rmfield(Dummy, 'cd');

    verifyError(testCase, @() af_tools.findcllinearrange(DummyAlpha), 'MATLAB:nonExistentField');
    verifyError(testCase, @() af_tools.findcllinearrange(DummyCl), 'MATLAB:nonExistentField');
    verifyError(testCase, @() af_tools.findcllinearrange(DummyCd), 'MATLAB:nonExistentField');

end

% ------------------------------------------------------------------------------
function test_noLinearRange(testCase)
    % Ensure that the function throws an exception if no linear range was found

    alpha = deg2rad(0:10)';
    c_l = rand(size(alpha));

    verifyError(testCase, @() af_tools.findcllinearrange(alpha, c_l), 'MATLAB:findcllinearrange:noRangeFound');

end

%% Test if function returns expected outputs

function test_correctResults(testCase)
    % Ensure that the function throws an exception if no linear range was found

    alpha = deg2rad(-10:10);
    c_l = 2 * pi * alpha;

    [alphaRange, clRange, clSlope] = af_tools.findcllinearrange(alpha, c_l);

    verifyEqual(testCase, alphaRange, [alpha(1), alpha(end)]');
    verifyEqual(testCase, clRange, [c_l(1), c_l(end)]');
    verifyEqual(testCase, clSlope, 2 * pi, 'AbsTol', 1e-9);
end
