% TEST_FORMATAIRFOILCOORD Unitary tests for the formatairfoilcoord function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_formatairfoilcoord

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase, ...
                @() af_tools.formatairfoilcoord(dummy, dummy, dummy, ...
                                                'autosave', true, 'overwrite', false, 'refine', true), ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidInputsType(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    Struct = struct;

    % 1. inputDir (char vector or string)
    verifyError(testCase, @() af_tools.formatairfoilcoord(empty), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(scal), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(logi), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(Struct), 'MATLAB:parsefileinputs:invalidType');

    % 2. inputFile (char, string, cell of chars)
    verifyError(testCase, @() af_tools.formatairfoilcoord(scal, empty), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(scal, scal), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(scal, logi), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(scal, Struct), 'MATLAB:parsefileinputs:invalidType');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    wrongName = 'wrongOption';
    testFiles = [pwd, '/test_utils/*'];

    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', true, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', true, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    empty = [];
    scal = 0;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    testFiles = [pwd, '/test_utils/*'];

    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', empty), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', scal), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', char), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', Struct), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'autosave', vect), ...
                'MATLAB:formatairfoilcoord:invalidType');

    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'overwrite', empty), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'overwrite', scal), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'overwrite', char), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'overwrite', Struct), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'overwrite', vect), ...
                'MATLAB:formatairfoilcoord:invalidType');

    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'refine', empty), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'refine', scal), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'refine', char), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'refine', Struct), ...
                'MATLAB:formatairfoilcoord:invalidType');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'refine', vect), ...
                'MATLAB:formatairfoilcoord:invalidType');

    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'outputFormat', empty), ...
                'MATLAB:formatairfoilcoord:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'outputFormat', scal), ...
                'MATLAB:formatairfoilcoord:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'outputFormat', char), ...
                'MATLAB:formatairfoilcoord:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'outputFormat', Struct), ...
                'MATLAB:formatairfoilcoord:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.formatairfoilcoord(testFiles, 'outputFormat', vect), ...
                'MATLAB:formatairfoilcoord:unrecognizedStringChoice');

end

% -------------------------------------
function test_fileNotFound(testCase)
    % Error if valid inputDir not found

    testDir = [pwd, '/test_utils/'];
    invalidDir = 'invaliddirectory';

    verifyWarning(testCase, @() af_tools.formatairfoilcoord({[testDir, 'clarky.dat'], [testDir, 'testdummy.dat']}), ...
                  'MATLAB:parsefileinputs:FileNotFound');
    % remove warning first warning about FileNotFound before it throws the error
    warning('off', 'MATLAB:parsefileinputs:FileNotFound');
    verifyError(testCase, @() af_tools.formatairfoilcoord([testDir, 'randomFile']), ...
                'MATLAB:parsefileinputs:noFilesFound');
    verifyError(testCase, @() af_tools.formatairfoilcoord(invalidDir), 'MATLAB:parsefileinputs:noFilesFound');

    warning on; % Reactivate warnings

end

%% Test if function returns expected outputs

function test_correctOrder(testCase)
    % Verify if the output starts and ends at X = 1;

    testFiles = [pwd, '/test_utils/*'];

    Dat = af_tools.formatairfoilcoord(testFiles);

    verifyEqual(testCase, Dat(1).coord(1, 1), 1);
    verifyEqual(testCase, Dat(1).coord(end, 1), 1);
    verifyEqual(testCase, Dat(2).coord(1, 1), 1);
    verifyEqual(testCase, Dat(2).coord(end, 1), 1);

end

function test_correctRefinePoints(testCase)
    % Verify if the output of refined solution has 101 points

    testFiles = [pwd, '/test_utils/*'];
    nPoints = 101;

    DatNoRefine = af_tools.formatairfoilcoord(testFiles);
    Dat = af_tools.formatairfoilcoord(testFiles, 'refine', true);

    for i = 1:length(Dat)
        if length(DatNoRefine(i).coord(:, 1)) > nPoints
            verifyEqual(testCase, Dat(i).coord(:, 1), DatNoRefine(i).coord(:, 1));
            verifyEqual(testCase, Dat(i).coord(:, 2), DatNoRefine(i).coord(:, 2));
        else
            verifyEqual(testCase, length(Dat(i).coord(:, 1)), nPoints);
            verifyEqual(testCase, length(Dat(i).coord(:, 2)), nPoints);
        end
    end

end
