% TEST_NACAAIRFOIL Unitary tests for the nacaairfoil function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_nacaairfoil

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    verifyError(testCase,  ...
                @() af_tools.nacaairfoil('0012', 100, 'spacing', 'halfcosine', 'zerote', true, 1),  ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidBaseInputs(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    % 1. Digits (char or str)
    verifyError(testCase, @() af_tools.nacaairfoil(empty), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil(scal), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil(logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil(Struct), 'MATLAB:parsenacainputs:invalidType');

    % 2. nPoints (numeric, above 2)
    verifyError(testCase, @() af_tools.nacaairfoil('0012', empty), 'MATLAB:parsenacainputs:expectedScalar');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', char), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', Struct), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', vect),  'MATLAB:parsenacainputs:expectedScalar');

end

function test_invalidDigits(testCase)
    % Error if digits is not an actual NACA 4 or 5 digit

    verifyError(testCase, @() af_tools.nacaairfoil('012'), 'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.nacaairfoil('012345'), 'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.nacaairfoil('oo12'), 'MATLAB:parsenacainputs:invalidDigits');
    verifyError(testCase, @() af_tools.nacaairfoil('24312'), 'MATLAB:parsenacainputs:invalid5Digits');

end

function test_warningLowNPoints(testCase)
    % Warns user if low number of points

    verifyWarning(testCase, @() af_tools.nacaairfoil('0012', 5, 'spacing', 'halfcosine'),  ...
                  'MATLAB:parsenacainputs:lowNPoints');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    wrongName = 'testspacing';

    verifyError(testCase, ...
                @() af_tools.nacaairfoil('0012', 100, wrongName),  ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, ...
                @() af_tools.nacaairfoil('0012', 100, wrongName, false), ...
                'MATLAB:InputParser:UnmatchedParameter');
    verifyError(testCase,  ...
                @() af_tools.nacaairfoil('0012', 100, 'spacing', 'cosine', wrongName),  ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase,  ...
                @() af_tools.nacaairfoil('0012', 100, 'spacing', 'cosine', wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', empty), ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', scal),  ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', logi), ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', char), ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', Struct),  ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'spacing', vect), ...
                'MATLAB:nacaairfoil:unrecognizedStringChoice');

    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'zerote', empty), 'MATLAB:nacaairfoil:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'zerote', scal), 'MATLAB:nacaairfoil:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'zerote', char), 'MATLAB:nacaairfoil:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'zerote', Struct), 'MATLAB:nacaairfoil:invalidType');
    verifyError(testCase, @() af_tools.nacaairfoil('0012', 100, 'zerote', vect), 'MATLAB:nacaairfoil:invalidType');

end

%% Test if function returns expected outputs

function test_correctNbCoordinates(testCase)
    % Test if output vector contain the correct amount of points
    % Note:
    %   nacaairfoil will always output an even number of points because:
    %       - the upper and lower surfaces will have the same number of points
    %       - the leading edge is counted twice (beginning and end of vector, so
    %         the loop is closed)
    %       - the trailing edge is counted once (at the middle)

    evenNPoints = 240;
    oddNPoints = 241;

    [coord4even] = af_tools.nacaairfoil('0012', evenNPoints);
    [coord5even] = af_tools.nacaairfoil('24012', evenNPoints);
    [coord4odd] = af_tools.nacaairfoil('0012', oddNPoints);
    [coord5odd] = af_tools.nacaairfoil('24012', oddNPoints);

    verifyEqual(testCase, size(coord4even, 1), evenNPoints + 1);
    verifyEqual(testCase, size(coord5even, 1), evenNPoints + 1);
    verifyEqual(testCase, size(coord4odd, 1), oddNPoints);
    verifyEqual(testCase, size(coord5odd, 1), oddNPoints);

end

function test_correctSpacing(testCase)
    % Test if xc follows the correct spacing rule

    nPoints = 222;

    linear = linspace(0, 1, ceil((nPoints + 1) / 2));
    betahalfcos = linspace(0, pi, ceil((nPoints + 1) / 2));
    halfcos = (1 - cos(betahalfcos)) / 2;
    betacos = linspace(0, pi / 2, ceil((nPoints + 1) / 2));
    cosine = 1 - cos(betacos);

    [coord_lin] = af_tools.nacaairfoil('0012', nPoints, 'spacing', 'linear');
    [coord_cos] = af_tools.nacaairfoil('0012', nPoints, 'spacing', 'cosine');
    [coord_half] = af_tools.nacaairfoil('0012', nPoints, 'spacing', 'halfcosine');

    % Test the two halves of the output vector (i.e. upper and lower surfaces)
    verifyEqual(testCase, coord_lin(1:(end + 1) / 2, 1), fliplr(linear)');
    verifyEqual(testCase, coord_cos(1:(end + 1) / 2, 1), fliplr(cosine)');
    verifyEqual(testCase, coord_half(1:(end + 1) / 2, 1), fliplr(halfcos)');
    verifyEqual(testCase, coord_lin(end:-1:(end + 1) / 2, 1), fliplr(linear)');
    verifyEqual(testCase, coord_cos(end:-1:(end + 1) / 2, 1), fliplr(cosine)');
    verifyEqual(testCase, coord_half(end:-1:(end + 1) / 2, 1), fliplr(halfcos)');

end

function test_correctDefaults(testCase)
    % Check if defaults are correctly set

    DEF_NPOINTS = 100;
    DEF_SPACING = 'halfcosine';
    DEF_ZEROTE = true;

    [coord] = af_tools.nacaairfoil('0012');
    [coord_def] = af_tools.nacaairfoil('0012', DEF_NPOINTS, 'spacing', DEF_SPACING, 'zerote', DEF_ZEROTE);

    verifyEqual(testCase, coord, coord_def);

end
