% TEST_NACACAMBER Unitary tests for the nacacamber function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_nacacamber

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    verifyError(testCase, @() af_tools.nacacamber('0012', 100, 'spacing', 'halfcosine', 1),  ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidBaseInputs(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    % 1. Digits (char or str)
    verifyError(testCase, @() af_tools.nacacamber(empty), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber(scal), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber(logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber(Struct), 'MATLAB:parsenacainputs:invalidType');

    % 2. nPoints (numeric, above 2)
    verifyError(testCase, @() af_tools.nacacamber('0012', empty), 'MATLAB:parsenacainputs:expectedScalar');
    verifyError(testCase, @() af_tools.nacacamber('0012', logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber('0012', char), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber('0012', Struct),  'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.nacacamber('0012', vect), 'MATLAB:parsenacainputs:expectedScalar');

end

function test_invalidDigits(testCase)
    % Error if digits is not an actual NACA 4 or 5 digit

    verifyError(testCase, @() af_tools.nacacamber('012'),  'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.nacacamber('012345'), 'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.nacacamber('oo12'), 'MATLAB:parsenacainputs:invalidDigits');
    verifyError(testCase, @() af_tools.nacacamber('24312'), 'MATLAB:parsenacainputs:invalid5Digits');

end

function test_warningLowNPoints(testCase)
    % Warns user if low number of points

    verifyWarning(testCase, @() af_tools.nacacamber('0012', 5, 'spacing', 'halfcosine'), ...
                  'MATLAB:parsenacainputs:lowNPoints');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    wrongName = 'testspacing';

    verifyError(testCase, @() af_tools.nacacamber('0012', 100, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, wrongName, false),  ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    optionName = 'spacing';
    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, empty), ...
                'MATLAB:nacacamber:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, scal),  ...
                'MATLAB:nacacamber:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, logi), ...
                'MATLAB:nacacamber:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, char),  ...
                'MATLAB:nacacamber:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, Struct),  ...
                'MATLAB:nacacamber:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.nacacamber('0012', 100, optionName, vect), ...
                'MATLAB:nacacamber:unrecognizedStringChoice');

end

%% Test if function returns expected outputs

function test_symmetricZeroCamber(testCase)
    % YC for symmetric airfoil should be 0 (no camber)

    [~, yc4] = af_tools.nacacamber('0012');
    [~, yc5] = af_tools.nacacamber('20012');
    expected_yc = zeros(1, 100);

    verifyEqual(testCase, yc4, expected_yc);
    verifyEqual(testCase, yc5, expected_yc);

end

function test_correctNbCoordinates(testCase)
    % Correct number of coordinates

    nPoints = 120;

    [xc4, yc4] = af_tools.nacacamber('0012', nPoints);
    [xc5, yc5] = af_tools.nacacamber('24012', nPoints);

    verifyEqual(testCase, length(xc4), nPoints);
    verifyEqual(testCase, length(yc4), nPoints);
    verifyEqual(testCase, length(xc5), nPoints);
    verifyEqual(testCase, length(yc5), nPoints);

end

function test_correctSpacing(testCase)
    % Test if xc follows the correct spacing rule

    nPoints = 222;

    linear = linspace(0, 1, nPoints);
    betahalfcos = linspace(0, pi, nPoints);
    halfcos = (1 - cos(betahalfcos)) / 2;
    betacos = linspace(0, pi / 2, nPoints);
    cosine = 1 - cos(betacos);

    [coord_lin] = af_tools.nacacamber('0012', nPoints, 'spacing', 'linear');
    [coord_cos] = af_tools.nacacamber('0012', nPoints, 'spacing', 'cosine');
    [coord_half] = af_tools.nacacamber('0012', nPoints, 'spacing', 'halfcosine');

    verifyEqual(testCase, coord_lin(:, 1), linear');
    verifyEqual(testCase, coord_cos(:, 1), cosine');
    verifyEqual(testCase, coord_half(:, 1), halfcos');

end

function test_correctDefaults(testCase)
    % Check if defaults are correctly set

    DEF_NPOINTS = 100;
    DEF_SPACING = 'halfcosine';

    [xc, yc] = af_tools.nacacamber('0012');
    [xc_def, yc_def] = af_tools.nacacamber('0012', DEF_NPOINTS, 'spacing', DEF_SPACING);

    verifyEqual(testCase, xc, xc_def);
    verifyEqual(testCase, yc, yc_def);

end

function test_correctMaxCamber(testCase)
    % Test if maxCamber is at the correct position

    naca4 = '6409';
    naca5 = '24012';

    % Get expected camber values and position
    maxCambVal4 = 0.06;
    maxCambPos4 = 0.4;
    maxCambPos5 = 0.2;

    % Calculate camberline
    [coord4] = af_tools.nacacamber(naca4, 100000, 'spacing', 'linear');
    [coord5] = af_tools.nacacamber(naca5, 100000, 'spacing', 'linear');

    % Get max camber value and position from camberline coordinates
    [maxVal4, maxPos4] = max(coord4(:, 2));
    [~, maxPos5] = max(coord5(:, 2));

    % Naca 4
    verifyEqual(testCase, maxVal4, maxCambVal4, 'RelTol', 1e-3);
    verifyEqual(testCase, coord4(maxPos4, 1), maxCambPos4, 'RelTol', 1e-3);

    % Naca 5
    verifyEqual(testCase, coord5(maxPos5, 1), maxCambPos5, 'RelTol', 1e-3);

end
