% TEST_XF2MAT Unitary tests for the xf2mat function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_xf2mat

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase, ...
                @() af_tools.xf2mat(dummy, dummy, dummy, 'autosave', true, 'trimAoas', false), ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidInputsType(testCase)
    % Error if base inputs are of the wrong type

    empty = [];
    scal = 0;
    logi = true;
    Struct = struct;

    % 1. inputDir (char vector or string)
    verifyError(testCase, @() af_tools.xf2mat(empty), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(scal), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(logi), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(Struct), 'MATLAB:parsefileinputs:invalidType');

    % 2. inputFile (char, string, cell of chars)
    verifyError(testCase, @() af_tools.xf2mat(scal, empty), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(scal, scal), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(scal, logi), 'MATLAB:parsefileinputs:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(scal, Struct), 'MATLAB:parsefileinputs:invalidType');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    wrongName = 'wrongOption';
    testdir = [pwd, '/test_utils/*'];

    verifyError(testCase, @() af_tools.xf2mat(testdir, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.xf2mat(testdir, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', true, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', true, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    empty = [];
    scal = 0;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    testdir = [pwd, '/test_utils/*'];

    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', empty), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', scal), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', char), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', Struct), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'autosave', vect), 'MATLAB:xf2mat:invalidType');

    verifyError(testCase, @() af_tools.xf2mat(testdir, 'trimAoas', empty), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'trimAoas', scal), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'trimAoas', char), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'trimAoas', Struct), 'MATLAB:xf2mat:invalidType');
    verifyError(testCase, @() af_tools.xf2mat(testdir, 'trimAoas', vect), 'MATLAB:xf2mat:invalidType');

end

% -------------------------------------
function test_fileNotFound(testCase)
    % Error if valid inputDir not found

    testdir = [pwd, '/test_utils/'];

    verifyWarning(testCase, @() af_tools.xf2mat({[testdir, 'xflr*'], [testdir, 'randomFile']}), ...
                  'MATLAB:parsefileinputs:FileNotFound');
    % Remove warning first warning about FileNotFound before it throws the error
    warning('off', 'MATLAB:parsefileinputs:FileNotFound');
    verifyError(testCase, @() af_tools.xf2mat([testdir, 'randomFile']), 'MATLAB:parsefileinputs:noFilesFound');
    warning on; % Reactivate warnings

end

%% Test if function returns expected outputs

function test_correctNbOfPolars(testCase)
    % Verify if the output has the correct number of polars (columns in arrays)

    testfiles = [pwd, '/test_utils/*'];

    Polar = af_tools.xf2mat(testfiles);

    verifyEqual(testCase, size(Polar.alpha, 2), 3);
    verifyEqual(testCase, size(Polar.alpha, 2), 3);
    verifyEqual(testCase, size(Polar.alpha, 2), 3);
    verifyEqual(testCase, size(Polar.alpha, 2), 3);

    verifyEqual(testCase, size(Polar.alpha, 2), size(Polar.cl, 2));
    verifyEqual(testCase, size(Polar.alpha, 2), size(Polar.cl, 2));
    verifyEqual(testCase, size(Polar.alpha, 2), size(Polar.cl, 2));

end

function test_correctDimensions(testCase)
    % Verify if the output has the correct number of polars (columns in arrays)

    testfiles = [pwd, '/test_utils/*'];

    Polar = af_tools.xf2mat(testfiles);

    verifyEqual(testCase, size(Polar.alpha, 1), size(Polar.cl, 1));
    verifyEqual(testCase, size(Polar.alpha, 1), size(Polar.cd, 1));
    verifyEqual(testCase, size(Polar.alpha, 1), size(Polar.cm, 1));

end

function test_correctPolarOrder(testCase)
    % Verify if the output is properly ordered by Reynolds

    testfile = [pwd, '/test_utils/xfoil*'];

    Polar = af_tools.xf2mat(testfile);
    import matlab.unittest.constraints.IsLessThanOrEqualTo
    verifyThat(testCase, Polar.reynolds(1), IsLessThanOrEqualTo(Polar.reynolds(2)));

end

function test_correctOutputValues(testCase)
    % Verify if the output has the correct values

    testfile = [pwd, '/test_utils/xfoil-naca2412_re1e6'];

    Polar = af_tools.xf2mat(testfile);

    verifyEqual(testCase, Polar.alpha(1), deg2rad(-20));
    verifyEqual(testCase, Polar.cl(1), -0.4490);
    verifyEqual(testCase, Polar.cd(1), 0.19678);
    verifyEqual(testCase, Polar.cm(1), 0.0070);
    verifyEqual(testCase, Polar.reynolds, 1e6);
    verifyEqual(testCase, Polar.mach, 0.1);
    verifyEqual(testCase, Polar.nCrit, 9);
    verifyEqual(testCase, Polar.origin, {'XFOIL'});
    verifyEqual(testCase, Polar.airfoil, {'NACA_2412'});

end

function test_autosave(testCase)
    % Verify if file is properly saved

    testfile = [pwd, '/test_utils/xfoil-naca2412_re1e6'];

    Polar = af_tools.xf2mat(testfile, 'autosave', true);
    savedPolar = load('NACA_2412-Re_1e6.mat');
    savedPolar = savedPolar.Polar;

    verifyEqual(testCase, Polar, savedPolar);

    % Cleanup
    delete NACA_2412-Re_1e6.mat;
end
